package cz.muni.fi.pa165.dao;

import cz.muni.fi.pa165.PersistenceSampleApplicationContext;
import cz.muni.fi.pa165.dao.team.TeamDao;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.enums.CountryEnum;
import org.assertj.core.api.Assertions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.testng.AbstractTransactionalTestNGSpringContextTests;
import org.testng.annotations.Test;

import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = PersistenceSampleApplicationContext.class)
public class TeamDaoTest extends AbstractTransactionalTestNGSpringContextTests {

    @Autowired
    private TeamDao teamDao;


    @Test
    void create() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaLeader");
        TeamEntity team = new TeamEntity("Autisti", new HashSet<>(List.of(teamCoachA)));
        teamDao.create(team);
        assertThat(team.getId()).isNotNull();
        Assertions.assertThat(teamDao.findAll()).hasSize(1);
    }

    @Test
    void findByIdExisting() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaLeader");
        TeamEntity team = new TeamEntity("Autisti", new HashSet<>(List.of(teamCoachA)));
        teamDao.create(team);

        var find = teamDao.findById(team.getId());
        assertThat(find).isEqualTo(team);
    }

    @Test
    void findByIdNonExisting() {
        Assertions.assertThat(teamDao.findAll()).isEmpty();

        var find = teamDao.findById(UUID.randomUUID().toString());
        assertThat(find).isNull();
    }

    @Test
    void findAll() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaA");
        TeamEntity teamA = new TeamEntity("Autisti I", new HashSet<>(List.of(teamCoachA)));
        TeamCoachEntity teamCoachB = new TeamCoachEntity("AutistaB");
        TeamEntity teamB = new TeamEntity("Autisti II", new HashSet<>(List.of(teamCoachB)));
        teamDao.create(teamA);
        teamDao.create(teamB);

        Assertions.assertThat(teamDao.findAll()).hasSize(2);
    }

    @Test
    void update() {
        TeamEntity team = new TeamEntity();
        teamDao.create(team);
        team.setName("AutistaC");
        team.setCountry(CountryEnum.BAHAMAS);
        teamDao.update(team);
        Assertions.assertThat(teamDao.findById(team.getId())).isEqualTo(team);
    }

    @Test
    void delete() {
        TeamCoachEntity teamCoachA = new TeamCoachEntity("AutistaA");
        TeamEntity teamA = new TeamEntity("Autisti I", new HashSet<>(List.of(teamCoachA)));
        TeamCoachEntity teamCoachB = new TeamCoachEntity("AutistaB");
        TeamEntity teamB = new TeamEntity("Autisti II", new HashSet<>(List.of(teamCoachB)));
        teamDao.create(teamA);
        teamDao.create(teamB);
        teamDao.remove(teamA);

        var teams = teamDao.findAll();
        Assertions.assertThat(teams).hasSize(1).contains(teamB);
    }


}
