package cz.muni.fi.pa165.services.teamCoach;

import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.entity.TournamentEntity;

import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
public interface TeamCoachService {

    void create(TeamCoachEntity teamCoach);

    List<TeamCoachEntity> findAll();

    TeamCoachEntity findById(String id);

    String update(TeamCoachEntity teamCoach);

    void remove(TeamCoachEntity teamCoach);

    /**
     * Signs team for the tournament. It is to be considered as an application to the tournament. Does not guarantee that team is automatically part
     * of tournament.
     * @param tournament TournamentEntity which team wants to attend.
     */
    void signForTournament(TeamCoachEntity teamCoach, TournamentEntity tournament);

    /**
     * Adds player who recently joined the team to the team.
     * @param newPlayer PlayerEntity that was the most recent addition to the team.
     */
    void addPlayerToTheTeam(TeamCoachEntity teamCoach, PlayerEntity newPlayer);

    /**
     * Removes player who recently left the team.
     * @param oldPlayer PlayerEntity that was the most recent fired from the team.
     */
    void removePlayerFromTheTeam(TeamCoachEntity teamCoach, PlayerEntity oldPlayer);

    /**
     * Adds coach who recently joined the team to the team.
     * @param newCoach TeamCoachEntity that was the most recent addition to the team.
     */
    void addCoachToTheTeam(TeamEntity team, TeamCoachEntity newCoach);

    /**
     * Removes coach who recently left the team.
     * @param teamCoach TeamCoachEntity that was the most recent fired from the team.
     */
    void removeCoachFromTheTeam(TeamEntity team, TeamCoachEntity teamCoach);
}
