package cz.muni.fi.pa165.services.tournament;

import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.entity.TournamentEntity;

import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
public interface TournamentService {
    /**
     * Get all tournaments played in a given city
     *
     * @param cityName string city name
     * @return all tournaments played in a given city
     */
    List<TournamentEntity> getTournamentsByCity(String cityName);

    /**
     * Get all tournaments won by a given team
     *
     * @param team string team name
     * @return all tournaments won by a given team
     */
    List<TournamentEntity> getTournamentsWonBy(TeamEntity team);

    void create(TournamentEntity tournament);

    List<TournamentEntity> findAll();

    TournamentEntity findById(String id);

    String update(TournamentEntity tournament);

    void remove(TournamentEntity tournament);

    /**
     * Adds team to the tournament if team is already signed for the event and capacity of an event allows it.
     * @param team Team to be added to the tournament.
     * @return Result of team' addition to the tournament.
     */
    boolean addTeamToTournament(TournamentEntity tournament, TeamEntity team);

    /**
     * Allows coach to sign team for the tournament.
     * @param team Team to be signed for the tournament.
     */
    void signTeamForTournament(TournamentEntity tournament, TeamEntity team);
}
