package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import cz.muni.fi.pa165.dao.match.MatchDao;
import cz.muni.fi.pa165.entity.MatchEntity;
import cz.muni.fi.pa165.services.match.MatchService;
import cz.muni.fi.pa165.services.match.MatchServiceImpl;
import org.hibernate.service.spi.ServiceException;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.UUID;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Adrian Piacek
 */
@ContextConfiguration(classes = ServiceConfiguration.class)
public class MatchServiceTest {
    @Mock
    private MatchDao matchDao;

    @Autowired
    @InjectMocks
    private MatchService matchService = new MatchServiceImpl();

    private MatchEntity matchA;

    @BeforeClass
    public void setup() throws ServiceException {
        MockitoAnnotations.openMocks(this);
    }

    @BeforeMethod
    public void createTestMatch() {
        this.matchA = new MatchEntity();
        matchA.setId(UUID.randomUUID().toString());
    }

    @Test
    public void createMatch() {
        matchService.create(matchA);

        verify(matchDao, times(1)).create(matchA);
    }

    @Test
    public void createNullMatch() {
        Assert.assertThrows(IllegalArgumentException.class, () -> matchService.create(null));
    }

    @Test
    public void findMatchById() {
        when(matchDao.findById(matchA.getId())).thenReturn(matchA);

        MatchEntity result = matchService.findById(matchA.getId());

        Assert.assertEquals(result.getId(), matchA.getId());
    }

    @Test
    public void findMatchByNullId() {
        Assert.assertThrows(IllegalArgumentException.class, () -> matchService.findById(null));
    }


    @Test
    public void findAllMatches() {
        MatchEntity matchB = new MatchEntity();
        when(matchDao.findAll()).thenReturn(asList(matchB, matchA));

        List<MatchEntity> result = matchService.findAll();

        Assert.assertEquals(result.size(), 2);
    }

    @Test
    public void deleteMatch() {
        matchService.remove(matchA);

        verify(matchDao, times(1)).remove(matchA);
    }

    @Test
    public void deleteNullMatch() {
        Assert.assertThrows(IllegalArgumentException.class, () -> matchService.remove(null));
    }

    @Test
    public void updateMatch() {
        MatchEntity matchB = new MatchEntity();
        matchService.update(matchB);

        verify(matchDao, times(1)).update(matchB);
    }

    @Test
    public void updateNullMatch() {
        Assert.assertThrows(IllegalArgumentException.class, () -> matchService.update(null));
    }

}
