package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import cz.muni.fi.pa165.dao.team.TeamDao;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.entity.TeamEntity;
import cz.muni.fi.pa165.services.team.TeamService;
import cz.muni.fi.pa165.services.team.TeamServiceImpl;
import org.hibernate.service.spi.ServiceException;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Adrian Piacek, Marek Kadlencik
 */
@ContextConfiguration(classes = ServiceConfiguration.class)
public class TeamServiceTest {
    @Mock
    private TeamDao teamDao;

    @Autowired
    @InjectMocks
    private TeamService teamService = new TeamServiceImpl();

    private TeamEntity teamA;

    @BeforeClass
    public void setup() throws ServiceException {
        MockitoAnnotations.openMocks(this);
    }

    @BeforeMethod
    public void createTestTeam() {
        TeamCoachEntity teamCoach = new TeamCoachEntity("Foo");
        this.teamA = new TeamEntity("Autisti I", new HashSet<>(List.of(teamCoach)));
        teamA.setId(UUID.randomUUID().toString());
    }

    @Test
    public void createTeam() {
        teamService.create(teamA);
        Mockito.verify(teamDao, Mockito.times(1)).create(teamA);
    }

    @Test
    public void createNullTeam() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamService.create(null));
    }

    @Test
    public void findTeamById() {
        when(teamDao.findById(teamA.getId())).thenReturn(teamA);

        TeamEntity result = teamService.findById(teamA.getId());

        Assert.assertEquals(result.getName(), teamA.getName());
    }

    @Test
    public void findTeamByNullId() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamService.findById(null));
    }


    @Test
    public void findAllTeams() {
        TeamEntity teamB = new TeamEntity();
        when(teamDao.findAll()).thenReturn(asList(teamB, teamA));

        List<TeamEntity> result = teamService.findAll();

        Assert.assertEquals(result.size(), 2);
    }

    @Test
    public void deleteTeam() {
        teamService.remove(teamA);

        verify(teamDao, times(1)).remove(teamA);
    }

    @Test
    public void deleteNullTeam() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamService.remove(null));
    }

    @Test
    public void updateTeam() {
        TeamCoachEntity teamCoach = new TeamCoachEntity("Booo");
        TeamEntity teamB = new TeamEntity("Autisti II", new HashSet<>(List.of(teamCoach)));

        teamService.update(teamB);

        verify(teamDao, times(1)).update(teamB);
    }

    @Test
    public void updateNullTeam() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamService.update(null));
    }
}
