package cz.muni.fi.pa165.mvc.controllers;

import cz.muni.fi.pa165.dto.PlayerDTO;
import cz.muni.fi.pa165.dto.TournamentDTO;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.facade.PlayerFacade;
import cz.muni.fi.pa165.facade.TournamentFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.util.UriComponentsBuilder;

import javax.validation.Valid;
import java.util.List;
import java.util.UUID;

/**
 * @author Adrian Piacek
 */
@Controller
@RequestMapping("/player")
public class PlayerController {

    @Autowired
    private PlayerFacade playerFacade;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public String list(Model model) {
        List<PlayerDTO> players = playerFacade.findAllPlayers();
        model.addAttribute("players", players);
        return "player/list";
    }

    @RequestMapping(value = "/new", method = RequestMethod.GET)
    public String newPlayer(Model model) {
        model.addAttribute("playerCreate", new PlayerDTO());
        return "player/new";
    }
    @ModelAttribute("countries")
    public CountryEnum[] countries() {
        return CountryEnum.values();
    }

    @RequestMapping(value = "/view/{id}", method = RequestMethod.GET)
    public String view(@PathVariable String id, Model model) {
        model.addAttribute("player", playerFacade.getPlayerById(id));
        return "player/view";
    }

    @RequestMapping(value = "/create", method = RequestMethod.POST)
    public String create(@Valid @ModelAttribute("playerCreate") PlayerDTO formBean, BindingResult bindingResult,
                         Model model, RedirectAttributes redirectAttributes, UriComponentsBuilder uriBuilder) {
        //in case of validation error forward back to the the form
        if (bindingResult.hasErrors()) {
            return "player/new";
        }
        //create product
        String id = playerFacade.createPlayer(formBean);
        //report success
        redirectAttributes.addFlashAttribute("alert_success", "Player " + id + " was created");
        return "redirect:" + uriBuilder.path("/player/view/{id}").buildAndExpand(id).encode().toUriString();
    }

    @RequestMapping(value = "/delete/{uuid}", method = RequestMethod.POST)
    public String delete(@PathVariable String uuid, UriComponentsBuilder uriBuilder, RedirectAttributes redirectAttributes) {
        PlayerDTO player = playerFacade.getPlayerById(uuid);
        try {
            playerFacade.delete(uuid);
            redirectAttributes.addFlashAttribute("alert_success", "Player \"" + player.getId() + "\" was deleted.");
        } catch (Exception ex) {
            redirectAttributes.addFlashAttribute("alert_danger", "Player \"" + player.getId() + "\" cannot be deleted.");
        }
        return "redirect:" + uriBuilder.path("/player/list").toUriString();
    }

}
